from enum import Enum
from typing import Any, Sequence, Optional
from pathlib import Path

from fastapi import APIRouter, FastAPI, Request
from pydantic import BaseSettings, Field
from sentry_sdk import init as sentry_init
from sentry_sdk.integrations.logging import LoggingIntegration

from . import fn
from .i18n import locales


class Env(Enum):
    """Env - for sentry commonly."""

    develop = 'develop'
    production = 'production'


class Config(BaseSettings):
    """Config."""

    secret: str

    service_host: str
    service_port: int

    service_name: str = 'Service'
    service_info: str = 'Service information'

    environment: Env = Field(default=Env.develop)
    debug: bool = Field(default_factory=fn.is_debug)
    testing: bool = Field(default_factory=fn.is_testing)

    routers: Sequence[APIRouter] = Field(default_factory=list)
    middleware: Sequence[Any] = Field(default_factory=list)

    sentry_dsn: Optional[str]
    log_config: Path = Field(default_factory=fn.log_config_path)

    locales: Sequence[str] = locales

    @staticmethod
    def create(
        routers: Sequence[APIRouter] = None,
        middleware: Sequence[Any] = None,
        **kwargs,
    ):
        """Create from env with runtime."""
        return Config(
            routers=routers if routers else [],
            middleware=middleware if middleware else [],
        )

    class Config:

        case_sensitive = False
        validate_all = True


index = APIRouter(tags=['Maintenance'])


@index.get('/status', status_code=200)
async def status(request: Request):
    """Maintenance service status."""
    return {'status': 'ready'}


def rpc_from_config(
    config: Config,
    **kwargs,
) -> FastAPI:
    """Create rpc from config"""
    # TODO: bind open doc schema for rpc
    api = FastAPI()

    api.config = config
    api.title = config.service_name
    api.description = config.service_info

    fn.setup_logger(config.log_config.as_posix(), debug=config.debug)

    from ._rpc import get_rpc_dispatcher

    dispatcher = get_rpc_dispatcher()

    if not config.testing:
        if config.sentry_dsn:
            sentry_init(
                dsn=config.sentry_dsn,
                server_name=config.service_name,
                release=fn.get_version(),
                environment=config.environment.value,
                attach_stacktrace=True,
                integrations=[LoggingIntegration()],
                request_bodies='always',
                with_locals=config.debug,
            )

    for middleware in config.middleware:
        api.add_middleware(middleware)

    api.include_router(dispatcher)

    return api


def from_config(
    config: Config,
    skip_index: bool = False,
    **kwargs,
) -> FastAPI:
    """Create api from config."""
    api = FastAPI(
        docs_url='/api/docs',
        redoc_url='/api/re_doc',
    )

    api.config = config

    api.title = config.service_name
    api.description = config.service_info

    fn.setup_logger(config.log_config.as_posix(), debug=config.debug)

    if not config.testing:
        if config.sentry_dsn:
            sentry_init(
                dsn=config.sentry_dsn,
                server_name=config.service_name,
                release=fn.get_version(),
                environment=config.environment.value,
                attach_stacktrace=True,
                integrations=[LoggingIntegration()],
                request_bodies='always',
                with_locals=config.debug,
            )

    for middleware in config.middleware:
        api.add_middleware(middleware)

    if not skip_index:
        api.include_router(index)

    for router in config.routers:
        api.include_router(router)

    return api
